<?php

/**
 * @file
 * Contains \Drupal\backup_migrate\Entity\Schedule.
 */

namespace Drupal\backup_migrate\Entity;

use BackupMigrate\Core\Exception\BackupMigrateException;
use BackupMigrate\Core\Main\BackupMigrateInterface;
use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Defines the Schedule entity.
 *
 * @ConfigEntityType(
 *   id = "backup_migrate_schedule",
 *   label = @Translation("Schedule"),
 *   module = "backup_migrate",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   handlers = {
 *     "list_builder" = "Drupal\backup_migrate\Controller\ScheduleListBuilder",
 *     "form" = {
 *       "default" = "Drupal\backup_migrate\Form\ScheduleForm",
 *       "delete" = "Drupal\backup_migrate\Form\ScheduleDeleteForm"
 *     },
 *   },
 *   links = {
 *     "edit-form" = "/admin/config/development/backup_migrate/schedule/edit/{backup_migrate_schedule}",
 *     "delete-form" = "/admin/config/development/backup_migrate/schedule/delete/{backup_migrate_schedule}",
 *     "collection" = "/admin/config/development/backup_migrate/schedule",
 *   },
 * )
 */
class Schedule extends ConfigEntityBase {
  /**
   * The Schedule ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The Schedule label.
   *
   * @var string
   */
  protected $label;

  /**
   * @param BackupMigrateInterface $bam
   *  The Backup and Migrate service object used to execute the backups
   *
   * @param bool $force
   *  Run the schedule even if it is not due to be run.
   */
  public function run(BackupMigrateInterface $bam, $force = FALSE) {

    $next_run_at = $this->getNextRun();
    $should_run_now = (REQUEST_TIME >= $next_run_at);
    if ($force || $should_run_now) {
      // Set the last run time before attempting backup.
      // This will prevent a failing schedule from retrying on every cron run.
      $this->setLastRun(REQUEST_TIME);

      try {
        \Drupal::logger('backup_migrate')->info("Running schedule %name", ['%name' => $this->get('label')]);
        // TODO: Set the config (don't just use the defaults).
        // Run the backup.
        $bam->backup($this->get('source_id'), $this->get('destination_id'));
        drupal_set_message('Backup Complete.');
      }
      catch (BackupMigrateException $e) {
        \Drupal::logger('backup_migrate')->error(
          "Error during scheduled backup (%name): %err",
          ['%name' => $this->get('label'), '%err' => $e->getMessage()]
        );
      }
    }
  }

  /**
   * @param $timestamp
   *  The unix time this schedule was last run.
   */
  public function setLastRun($timestamp) {
    \Drupal::keyValue('backup_migrate_schedule:last_run')->set($this->id(), $timestamp);
  }

  /**
   * @return int $timestamp
   *  The unix time this schedule was last run.
   */
  public function getLastRun() {
    return \Drupal::keyValue('backup_migrate_schedule:last_run')->get($this->id());
  }

  /**
   * Get the next time this schedule should run.
   * @return int
   */
  public function getNextRun() {
    $last_run_at = $this->getLastRun();
    if ($last_run_at) {
      return $last_run_at + $this->get('period');
    }
    return REQUEST_TIME - 1;
  }

  /**
   * Return the schedule frequency formatted for display in human language
   *
   * @return \Drupal\Core\StringTranslation\PluralTranslatableMarkup
   * @throws \BackupMigrate\Core\Exception\BackupMigrateException
   */
  public function getPeriodFormatted() {
    return Schedule::formatPeriod(Schedule::secondsToPeriod($this->get('period')));
  }

  /**
   * Convert a number of of seconds into a period array.
   *
   * @param int $seconds
   * @return array An array containing the period definition and the number of them.
   *  ['units' => 123, 'type' => [...]]
   * @throws \BackupMigrate\Core\Exception\BackupMigrateException
   */
  protected static function secondsToPeriod($seconds) {
    foreach (array_reverse(Schedule::getPeriodTypes()) as $type) {
      if (($seconds % $type['seconds']) === 0) {
        return ['units' => $seconds / $type['seconds'], 'type' => $type];
      }
    }

    throw new BackupMigrateException('Invalid period.');
  }

  /**
   * Convert a period array into seconds.
   *
   * @param array $period A period array
   * @return mixed
   * @throws \BackupMigrate\Core\Exception\BackupMigrateException
   */
  protected static function periodToSeconds($period) {
    return $period['units'] * $period['type']['seconds'];
  }

  /**
   * Convert a period array into seconds.
   *
   * @param $period
   * @return \Drupal\Core\StringTranslation\PluralTranslatableMarkup
   */
  protected static function formatPeriod($period) {
    return \Drupal::translation()->formatPlural($period['units'], $period['type']['singular'], $period['type']['plural']);
  }

  /**
   * Get a list of available backup periods. Only returns time periods which have a
   *  (reasonably) consistent number of seconds (ie: no months).
   *
   * @return array
   */
  protected static function getPeriodTypes() {
    return array(
      'seconds' => array('type' => 'seconds', 'seconds' => 1, 'title' => 'Seconds', 'singular' => 'Once a second', 'plural' => 'Every @count seconds'),
      'minutes' => array('type' => 'minutes', 'seconds' => 60, 'title' => 'Minutes', 'singular' => 'Once a minute', 'plural' => 'Every @count minutes'),
      'hours' => array('type' => 'hours', 'seconds' => 3600, 'title' => 'Hours', 'singular' => 'Hourly', 'plural' => 'Every @count hours'),
      'days' => array('type' => 'days', 'seconds' => 86400, 'title' => 'Days', 'singular' => 'Daily', 'plural' => 'Every @count days'),
      'weeks' => array('type' => 'weeks', 'seconds' => 604800, 'title' => 'Weeks', 'singular' => 'Weekly', 'plural' => 'Every @count weeks'),
    );
  }

}
