<?php

/**
 * @file
 * Contains \Drupal\cforge_import\Plugin\CsvParser\Users.
 *
 */
namespace Drupal\cforge_import\Plugin\CsvParser;

use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;


abstract class Users extends ImportBase implements ContainerFactoryPluginInterface{

  protected $siteSettings;

  function __construct($configuration, $plugin_id, $plugin_definition, $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->siteSettings = $config_factory->get('system_site');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function deleteAll() {
    foreach (\Drupal::entityTypeManager()->getStorage('user')->loadMultiple() as $entity) {
      if ($entity->id() < 2) {
        continue;//don't delete user 1
      }
      $entity->delete();
    }
  }

  /**
   * {@inheritdoc}
   */
  protected static function createEntity() {
    return \Drupal::entityTypeManager()->getStorage('user')->create()->activate();
  }

  /**
   * {@inheritdoc}
   */
  public static function columns() {
    return [
      'name' => t('user/login name'),
      'mail' => t('email address'),
      'password' => t('password (optional)'),
      'roles' => t('name of the role, e.g. trader, committee'),
      'firstname' => t('first name'),
      'lastname' => t('last name'),
      'created' => t('created date in any parsable php strtotime format'),
      'access' => t('last login time in any parsable php strtotime format'),
      'portrait' => t('absolute url of image file'),
      'address.line1' => t('address line 1'),
      'address.line2' => t('address line 2'),
      'address.organization' => t('organisation name'),
      'address.dependent_locality' => t('Neighbourhood name (N.B references node type. REQUIRED)'),
      'address.locality' => t('City/town'),
      'address.administrative_area' => t('Region/Province'),
      //'address.postal_code', we have  no way to force these to be valid
      'address.country_code' => t('2 digit country code (default is @default)', ['@default' => 'blah']),
      'phones' => t('mobile phone number'),
      'phones2' => t('other phone number'),
      'notes' => t("'about me' text in html"),
      'notes_admin' => t('plaintext notes visible only to committee'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  function makeBatch($rows, $delete = FALSE, $test = TRUE) {
    $batch = parent::makeBatch($rows, $delete, $test);
    $batch['title'] = 'Importing users';
    return $batch;
  }

  static function nameProcess(UserInterface $account, $val, $allfields) {
    if (static::required($val)) {
       $account->setUsername($val);
    }
    else {
      return 'missing username';
    }
  }

  static function passwordProcess(UserInterface $account, $val, $allfields) {
    $account->setPassword($val);
  }

  static function mailProcess(UserInterface $account, $val, $allfields) {
    if (empty($val) or !preg_match('/^.+\@\S+\.\S+$/', $val)) {
      $sitemail = \Drupal::config('system.site')->get('mail');
      $suffix = substr($sitemail, strpos($sitemail, '@'));
      if (!$suffix) {
        throw new Exception('Site mail cannot be used for creating dummy emails: '.$sitemail);
      }
      $val = $allfields['name'] . $suffix;
    }
    $account->setEmail($val);
  }

  static function coordinatorProcess(UserInterface $account, $val, &$allfields) {
    $account->addRole('superneighbour');
  }

  static function address_dependent_localityProcess(UserInterface $account, $val, $allfields) {
    //temporary - a dependent locality will be a group later on and thus the address will reference it
    static::required($val, 'Brisbane');
    if (!\Drupal::entityQuery('node')->condition('type', 'neighbourhood')->condition('title', $val)->execute()) {
      \Drupal\node\Entity\Node::create(['type' => 'neighbourhood', 'title' => $val])->save();
    }
    $account->address->dependent_locality = $val;
  }

  static function phones2Process(UserInterface $account, $val, $allfields) {
    $vals = [$allfields['phones'], $val];
    $account->set('phones', $vals);
  }

  static function notesProcess(UserInterface $account, $val, $allfields) {
    $val = str_replace('<br />', '\n', $val);
    $account->set('notes', $val);
  }

  static function portraitProcess(UserInterface $account, $src, $allfields) {
    if ($src) {
      $info = pathinfo($src);
      $newpath = 'private://portraits/'.$info['basename'];
      if (copy($src, $newpath)) {
        $file = \Drupal\file\Entity\File::create(['uri' => $newpath, 'uid' => $account->id()]);
        $file->save();
        $account->portrait->target_id = $file->id();
        //the portrait will fail validation because the attachment to the entity is not saved until postsave phase.
      }
    }
  }

  //batch callback.
  //the sandbox seems to be empty every time, unfortunately
  static function saveEntities($rows, $test, &$sandbox) {
    $default_country = \Drupal::config('system.date')->get('country.default');
    foreach ($rows as $key => &$row) {
      if (empty($row['password'])) {
        unset($row[$key]);
      }
      if (empty($row['address.country_code'])) {
        $row['address.country_code'] = $default_country;
      }
    }
    parent::saveEntities($rows, $test, $sandbox);
  }

}
