<?php

/**
 * @file
 * Contains \Drupal\digest\Controller\RemoteLogin.
 */

namespace Drupal\digest\Controller;

use Drupal\Component\Utility\Crypt;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\user\UserDataInterface;
use Drupal\user\UserStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Controller routines for user routes.
 */
class RemoteLogin extends ControllerBase {

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The user storage.
   *
   * @var \Drupal\user\UserStorageInterface
   */
  protected $userStorage;

  /**
   * The user data service.
   *
   * @var \Drupal\user\UserDataInterface
   */
  protected $userData;

  /**
   * Constructs a UserController object.
   *
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\user\UserStorageInterface $user_storage
   *   The user storage.
   * @param \Drupal\user\UserDataInterface $user_data
   *   The user data service.
   */
  public function __construct(DateFormatterInterface $date_formatter, UserStorageInterface $user_storage, UserDataInterface $user_data) {
    $this->dateFormatter = $date_formatter;
    $this->userStorage = $user_storage;
    $this->userData = $user_data;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('date.formatter'),
      $container->get('entity.manager')->getStorage('user'),
      $container->get('user.data')
    );
  }

  /**
   * Returns the user password reset page.
   *
   * @param int $uid
   *   UID of user requesting reset.
   * @param int $timestamp
   *   The current timestamp.
   * @param string $hash
   *   Login link hash.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   The form structure or a redirect response.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException
   *   If the login link is for a blocked user or invalid user ID.
   */
  public function page($uid, $timestamp, $hash) {
    $account = $this->currentUser();
    $redirect = FALSE;
    $incoming_user = $this->userStorage->load($uid);
    $config = $this->config('digest.settings');
    // When processing the one-time login link, we have to make sure that a user
    // isn't already logged in.
    if ($account->isAuthenticated()) {
      // The current user is already logged in.
      if ($account->id() == $uid) {
        $redirect = TRUE;
      }
      elseif ($incoming_user) {
        user_logout();//so we can log in as the incoming user later
        return $this->redirect('<front>');
      }
    }

    // Verify that the user exists and is active.
    if (!$redirect && $incoming_user && $incoming_user->isActive()) {
      $timeout = $config->get('remote_login_timeout');
      if ($incoming_user->isAuthenticated()) {
        if ($timestamp >= $incoming_user->getLastLoginTime()) {
          if (REQUEST_TIME - $timestamp < $timeout) {
            if ($timestamp <= REQUEST_TIME) {
              if (Crypt::hashEquals($hash, user_pass_rehash($user, $timestamp))) {
                $redirect = TRUE;
              }
            }
          }
        }
      }
    }
    if ($redirect) {
      return $this->redirect('digest.settings.user', ['user' => $uid]);
    }
    elseif($incoming_user) {
      drupal_set_message($this->t('You have tried to use a one-time login link that in invalid or has expired. You will need to log in normally'), 'error');
      return $this->redirect('user.login');
    }
    // Blocked or invalid user ID, so deny access. The parameters will be in the
    // watchdog's URL for the administrator to check.
    throw new AccessDeniedHttpException();
  }

  /**
   * test the digest for the current user and the last month
   */
  public function test() {
    $recipient = \Drupal\user\Entity\User::load(\Drupal::currentUser()->id());
    $user_data = \Drupal::service('user.data');
    $settings = $user_data->get('digest', $recipient->id(), 'digest');
    if ($settings) {
      //taken from the QueueWorker
      \Drupal::service('plugin.manager.mail')
        ->mail('digest',
          'digest',
          $recipient->getEmail(),
          $recipient->getPreferredLangcode(),
          [
            'user' => $recipient,
            'displays'  => $settings['displays'],
            'since' => strtotime('-1 month')
          ]
        );
      $message = 'sent';
    }
    else {
      drupal_set_message("The current user has no digest settings. See user/".$recipient->id()."digest");
      $message = 'not sent';
    }
    return ['#markup' => $message];
  }
}
